/*
 
 *****************************************************************************
 * Author:                                                                   *
 * ------                                                                    *
 *  Anton Kokalj                                  Email: Tone.Kokalj@ijs.si  *
 *  Department of Physical and Organic Chemistry  Phone: x 386 1 477 3523    *
 *  Jozef Stefan Institute                          Fax: x 386 1 477 3811    *
 *  Jamova 39, SI-1000 Ljubljana                                             *
 *  SLOVENIA                                                                 *
 *                                                                           *
 * Source: $XCRYSDEN_TOPDIR/C/writeXSF.c
 * ------                                                                    *
 * Copyright (c) 1996-2003 by Anton Kokalj                                   *
 *****************************************************************************

*/

#include <stdio.h>
#include <tcl.h>
#include "struct.h"
#include "xcfunc.h"

/* writeXSF.c */
static void _writeMolecule(FILE *fp);
static void _writePeriodic(FILE *fp);
static void _printAtom(FILE *fp, int nat, double x, double y, double z, double *f);

/*
 * usage: xc_writeXSF filename 
 */
int
XC_WriteXSFCmd(ClientData clientData, Tcl_Interp *interp,
	       int argc, char *argv[])
{
  FILE *fp;

  if ( argc != 2 ) {
    sprintf(interp->result, "wrong # of arguments: should be \"xc_writeXSF filename\"");
    return TCL_ERROR;
  }

  fp = fopen(argv[1], "w");
  if (!fp) {
    sprintf(interp->result, "couldn't write file %s", argv[1]);
    return TCL_ERROR;
  }

  if ( xcr.dim == 0 ) {
    /* MOLECULE */
    _writeMolecule(fp);
  } else {
    /* PERIODIC STRUCTURE */
    _writePeriodic(fp);
  }

  fclose(fp);
  return TCL_OK;
}

static void _writeMolecule(FILE *fp) {
  register int ia;  
  
  fprintf(fp, "ATOMS\n");
  for (ia=1; ia<natoms; ia++)
    _printAtom(fp, nat[ia], xat[ia], yat[ia], zat[ia], fv[ia]);
}

static void _writePeriodic(FILE *fp) {
  register int ia, ic;  
  double       xyz[3];

  /* write periodic structure type */

  if      ( xcr.dim == 3 ) fprintf(fp, "CRYSTAL\n");
  else if ( xcr.dim == 2 ) fprintf(fp, "SLAB\n");
  else                     fprintf(fp, "POLYMER\n");

  /* write PRIMVEC */

  fprintf(fp, "PRIMVEC\n");
  for (ia=0; ia<3; ia++)
    fprintf(fp, "%15.10f  %15.10f  %15.10f\n",
	    vec.prim[ia][0], vec.prim[ia][1], vec.prim[ia][2]);

  /* write CONVVEC */

  if ( xcr.lconvvec ) 
    {
      fprintf(fp, "CONVVEC\n");
      for (ia=0; ia<3; ia++)
	fprintf(fp, "%15.10f  %15.10f  %15.10f\n",
		vec.conv[ia][0], vec.conv[ia][1], vec.conv[ia][2]);
    }          

  /* write PRIMCOORD */
  
  fprintf(fp, "PRIMCOORD\n");
  fprintf(fp, "%d 1\n", xcr.natr);
  for (ia = 0; ia < xcr.natr; ia++) 
    {
      for (ic = 0; ic < 3; ic++)   
	xyz[ic] = 
	  vec.prim[0][ic] * xcr.prim_fcoor[ia][0] +
	  vec.prim[1][ic] * xcr.prim_fcoor[ia][1] +
	  vec.prim[2][ic] * xcr.prim_fcoor[ia][2];	  
      fprintf(fp, "%3d    %15.10f  %15.10f  %15.10f\n",
	      xcr.prim_nat[ia], xyz[0], xyz[1], xyz[2]);
    }
}

static void _printAtom(FILE *fp, 
		       int nat, double x, double y, double z, double *f) {
  if ( xcr.lforce ) {
    fprintf(fp, "%3d    %15.10f  %15.10f  %15.10f\n", nat, x, y, z);
  } else {
    fprintf(fp, "%3d    %15.10f  %15.10f  %15.10f    %15.10f  %15.10f  %15.10f\n", nat, x, y, z, f[0], f[1], f[2]);
  }
}
